@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2014 by Tomasz Bojczuk                                  *
- *   tomaszbojczuk@gmail.com                                               *
+ *   Copyright (C) 2014-2015 by Tomasz Bojczuk                             *
+ *   seelook@gmail.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,5 +19,5 @@
 
 #include "tpath.h"
 
-QString Tpath::main = "";
+QString Tpath::main = QString();
 
