@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2013-2015 by Tomasz Bojczuk                             *
- *   tomaszbojczuk@gmail.com                                               *
+ *   Copyright (C) 2013-2017 by Tomasz Bojczuk                             *
+ *   seelook@gmail.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -16,118 +16,121 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
  ***************************************************************************/
 
-
 #include "trtaudio.h"
 #include <taudioparams.h>
-#include <QStringList>
-#include <QDebug>
-#include <QFileInfo>
+#include <QtCore/qstringlist.h>
+#include <QtCore/qfileinfo.h>
+#include <QtCore/qdebug.h>
 
 
 /*static*/
-RtAudio* 													TrtAudio::m_rtAduio = 0;
-RtAudio::StreamParameters* 				TrtAudio::m_inParams = 0;
-RtAudio::StreamParameters* 				TrtAudio::m_outParams = 0;
-RtAudio::StreamOptions* 					TrtAudio::streamOptions = 0;
-quint32 													TrtAudio::m_sampleRate = 44100;
-quint32                           TrtAudio::m_inSR = 48000;
-quint32                           TrtAudio::m_outSR = 44100;
-unsigned int 											TrtAudio::m_bufferFrames = 1024;
-bool 															TrtAudio::m_isAlsaDefault = false;
-QString 													TrtAudio::m_inDevName = "anything";
-QString 													TrtAudio::m_outDevName = "anything";
-TrtAudio::callBackType				 		TrtAudio::m_cbIn = 0;
-TrtAudio::callBackType 						TrtAudio::m_cbOut = 0;
-TaudioObject*											TrtAudio::m_ao = 0;
-RtAudioCallback										TrtAudio::m_callBack = TrtAudio::duplexCallBack;
-bool 															TrtAudio::m_JACKorASIO = false;
-bool 															TrtAudio::forceUpdate = false;
-TrtAudio::EaudioState             TrtAudio::m_state = TrtAudio::e_iddle;
-bool                              TrtAudio::m_areSplit = true;
+RtAudio*                           TrtAudio::m_rtAduio = 0;
+RtAudio::StreamParameters*         TrtAudio::m_inParams = 0;
+RtAudio::StreamParameters*         TrtAudio::m_outParams = 0;
+RtAudio::StreamOptions*            TrtAudio::streamOptions = 0;
+quint32                            TrtAudio::m_sampleRate = 44100;
+quint32                            TrtAudio::m_inSR = 48000;
+quint32                            TrtAudio::m_outSR = 44100;
+unsigned int                       TrtAudio::m_bufferFrames = 1024;
+bool                               TrtAudio::m_isAlsaDefault = false;
+QString                            TrtAudio::m_inDevName = "anything";
+QString                            TrtAudio::m_outDevName = "anything";
+TrtAudio::callBackType             TrtAudio::m_cbIn = 0;
+TrtAudio::callBackType             TrtAudio::m_cbOut = 0;
+TaudioObject*                      TrtAudio::m_ao = 0;
+RtAudioCallback                    TrtAudio::m_callBack = TrtAudio::duplexCallBack;
+bool                               TrtAudio::m_JACKorASIO = false;
+bool                               TrtAudio::forceUpdate = false;
+bool                               TrtAudio::m_sendPlayingFinished = false;
+TrtAudio::EaudioState              TrtAudio::m_state = TrtAudio::e_idle;
+bool                               TrtAudio::m_areSplit = true;
 
-int                               m_preferredBF = 512;
-bool                              m_audioUpdated = false;
+int                                m_preferredBF = 512;
+bool                               m_audioUpdated = false;
 
 
 void TrtAudio::createRtAudio() {
-	if (m_rtAduio == 0) { // Create RtAudio instance if doesn't exist
+  if (m_rtAduio == 0) { // Create RtAudio instance if doesn't exist
 #if defined(Q_OS_WIN)
-		RtAudio::Api rtAPI;
-		if ((int)QSysInfo::windowsVersion() < (int)QSysInfo::WV_VISTA)
-			rtAPI = RtAudio::WINDOWS_DS; // Direct sound for older versions
-		else // WASAPI since Vista
-			rtAPI = RtAudio::WINDOWS_WASAPI;
-		if (m_JACKorASIO)
-			rtAPI = RtAudio::UNSPECIFIED; // ASIO has a priority
-#else
-		RtAudio::Api rtAPI;
-		if (m_JACKorASIO)
+    RtAudio::Api rtAPI;
+    if ((int)QSysInfo::windowsVersion() < (int)QSysInfo::WV_VISTA)
+      rtAPI = RtAudio::WINDOWS_DS; // Direct sound for older versions
+    else // WASAPI since Vista
+      rtAPI = RtAudio::WINDOWS_WASAPI;
+    if (m_JACKorASIO)
+      rtAPI = RtAudio::UNSPECIFIED; // ASIO has a priority
+#elif defined(Q_OS_LINUX)
+    RtAudio::Api rtAPI;
+    if (m_JACKorASIO)
       rtAPI = RtAudio::UNSPECIFIED; // check is JACK possible and allow it
     else
       rtAPI = RtAudio::LINUX_ALSA; // force ALSA
-#endif
-#if defined(__LINUX_PULSE__)
+    #if defined(__LINUX_PULSE__)
       QFileInfo pulseBin("/usr/bin/pulseaudio");
       if (!m_JACKorASIO && pulseBin.exists()) // we check is PA possible to run - without check, it can hang over.
-				rtAPI = RtAudio::LINUX_PULSE;
-#endif
+        rtAPI = RtAudio::LINUX_PULSE;
+    #endif
+#else
+    RtAudio::Api rtAPI = RtAudio::MACOSX_CORE;
+ #endif
+
     try {
-      m_rtAduio = new RtAudio(rtAPI);  
+      m_rtAduio = new RtAudio(rtAPI);
       m_rtAduio->showWarnings(false);
     } catch (RtAudioError& e) {
-      qDebug() << "Cannot create RtAudio instance" << QString::fromStdString(e.getMessage());
+      qDebug() << "[TrtAudio] Cannot create RtAudio instance" << QString::fromStdString(e.getMessage());
       m_rtAduio = 0;
     }
-	}
+  }
 }
 
 
 QString TrtAudio::currentRtAPI() {
-	QString rtApiTxt;
-	if (m_rtAduio) {
-		switch (getCurrentApi()) {
-			case RtAudio::WINDOWS_DS:
-				rtApiTxt = "Direct Sound";
-				break;
-			case RtAudio::WINDOWS_WASAPI:
-				rtApiTxt = "WAS API";
-				break;
-			case RtAudio::WINDOWS_ASIO:
-				rtApiTxt = "ASIO";
-				break;
-			case RtAudio::LINUX_ALSA:
-				rtApiTxt = "ALSA";
-				break;
-			case RtAudio::UNIX_JACK:
-				rtApiTxt = "JACK";
-				break;
-			case RtAudio::LINUX_PULSE:
-				rtApiTxt = "pulseaudio";
-				break;
-			case RtAudio::RtAudio::MACOSX_CORE:
-				rtApiTxt = "CoreAudio";
-				break;
-			default:
-				rtApiTxt = "Undefined";
-				break;
-		}
-	} else
-			rtApiTxt = "RtAudio API doesn't exist";
-	return rtApiTxt;
+  QString rtApiTxt;
+  if (m_rtAduio) {
+    switch (getCurrentApi()) {
+      case RtAudio::WINDOWS_DS:
+        rtApiTxt = "Direct Sound";
+        break;
+      case RtAudio::WINDOWS_WASAPI:
+        rtApiTxt = "WAS API";
+        break;
+      case RtAudio::WINDOWS_ASIO:
+        rtApiTxt = "ASIO";
+        break;
+      case RtAudio::LINUX_ALSA:
+        rtApiTxt = "ALSA";
+        break;
+      case RtAudio::UNIX_JACK:
+        rtApiTxt = "JACK";
+        break;
+      case RtAudio::LINUX_PULSE:
+        rtApiTxt = "pulseaudio";
+        break;
+      case RtAudio::RtAudio::MACOSX_CORE:
+        rtApiTxt = "CoreAudio";
+        break;
+      default:
+        rtApiTxt = "Undefined";
+        break;
+    }
+  } else
+      rtApiTxt = "RtAudio API doesn't exist";
+  return rtApiTxt;
 }
 
 
 
 #if !defined (Q_OS_MAC)
 void TrtAudio::setJACKorASIO(bool jack) {
-	if (jack != m_JACKorASIO) {
-		abortStream();
+  if (jack != m_JACKorASIO) {
+    abortStream();
     closeStream();
-		delete m_rtAduio;
-		m_rtAduio = 0;
-		m_JACKorASIO = jack;
-		createRtAudio();
-	}
+    delete m_rtAduio;
+    m_rtAduio = 0;
+    m_JACKorASIO = jack;
+    createRtAudio();
+  }
 }
 
 
@@ -152,7 +155,7 @@ void TrtAudio::printSupportedFormats(RtAudio::DeviceInfo& devInfo) {
     fmt += " RTAUDIO_FLOAT32";
   if (devInfo.nativeFormats & 0x20)
     fmt += " RTAUDIO_FLOAT64";
-  qDebug() << "supported sample formats:" << fmt;
+  qDebug() << "[TrtAudio] supported sample formats:" << fmt;
 }
 
 
@@ -160,7 +163,7 @@ void TrtAudio::printSupportedSampleRates(RtAudio::DeviceInfo& devInfo) {
   QString sRates;
   for (int i = 0; i < devInfo.sampleRates.size(); i++)
     sRates += QString("%1 ").arg(devInfo.sampleRates.at(i));
-  qDebug() << "supported sample rates:"<< sRates;
+  qDebug() << "[TrtAudio] supported sample rates:"<< sRates;
 }
 
 
@@ -168,42 +171,42 @@ void TrtAudio::printSupportedSampleRates(RtAudio::DeviceInfo& devInfo) {
 
 
 TrtAudio::TrtAudio(TaudioParams* audioP, TrtAudio::EdevType type, TrtAudio::callBackType cb) :
-	m_audioParams(audioP),
-	m_type(type)
+  m_audioParams(audioP),
+  m_type(type)
 {
-	if (m_type == e_input) {
-		m_cbIn = cb;
-		if (!m_inParams)
-			m_inParams = new RtAudio::StreamParameters;
-	} else {
-		m_cbOut = cb;
-		if (!m_outParams)
-			m_outParams = new RtAudio::StreamParameters;
-	}
-	if (!streamOptions) {
-			streamOptions = new RtAudio::StreamOptions;
-			streamOptions->streamName = "Nootka";
-			m_ao = new TaudioObject();
-	}
-	ao()->blockSignals(true);
-	createRtAudio();
-	updateAudioParams();
-	ao()->blockSignals(false);
+  if (m_type == e_input) {
+    m_cbIn = cb;
+    if (!m_inParams)
+      m_inParams = new RtAudio::StreamParameters;
+  } else {
+    m_cbOut = cb;
+    if (!m_outParams)
+      m_outParams = new RtAudio::StreamParameters;
+  }
+  if (!streamOptions) {
+      streamOptions = new RtAudio::StreamOptions;
+      streamOptions->streamName = "Nootka";
+      m_ao = new TaudioObject();
+  }
+  ao()->blockSignals(true);
+  createRtAudio();
+  updateAudioParams();
+  ao()->blockSignals(false);
 }
 
 
 TrtAudio::~TrtAudio()
 {
-// 	closeStream();
-	if (m_outParams == 0 && m_inParams == 0) {
-		delete m_rtAduio;
-		m_rtAduio = 0;
-		delete streamOptions;
-		streamOptions = 0;
-		delete m_ao;
-		m_ao = 0;
-	}
-		
+//   closeStream();
+  if (m_outParams == 0 && m_inParams == 0) {
+    delete m_rtAduio;
+    m_rtAduio = 0;
+    delete streamOptions;
+    streamOptions = 0;
+    delete m_ao;
+    m_ao = 0;
+  }
+
 }
 
 
@@ -212,10 +215,12 @@ void TrtAudio::updateAudioParams() {
   if (forceUpdate || (m_inParams && m_inDevName != audioParams()->INdevName) || (m_outParams && m_outDevName != m_audioParams->OUTdevName) ||
       (audioParams()->forwardInput && m_callBack == &duplexCallBack) || (!audioParams()->forwardInput && m_callBack == &passInputCallBack) ) {
     closeStream();
+#if defined (Q_OS_LINUX) || defined (Q_OS_WIN)
     setJACKorASIO(audioParams()->JACKorASIO);
-		forceUpdate = false; // no more
-		m_audioUpdated = true;
-		if (audioParams()->forwardInput || getCurrentApi() == RtAudio::WINDOWS_ASIO) {
+#endif
+    forceUpdate = false; // no more
+    m_audioUpdated = true;
+    if (audioParams()->forwardInput || getCurrentApi() == RtAudio::WINDOWS_ASIO) {
       m_areSplit = false;
     } else {
       m_areSplit = true;
@@ -248,7 +253,7 @@ void TrtAudio::updateAudioParams() {
               RtAudio::DeviceInfo inputInfo;
               getDeviceInfo(inputInfo, inDevId);
               if (inputInfo.inputChannels <= 0) {
-                qDebug("wrong default input device");
+                qDebug("[TrtAudio] wrong default input device");
                 deleteInParams();
               }
             }
@@ -261,7 +266,7 @@ void TrtAudio::updateAudioParams() {
           getDeviceInfo(outInfo, outDevId);
           if (outDevId > -1) {
             if (outInfo.outputChannels <= 0) {
-              qDebug("wrong default output device");
+              qDebug("[TrtAudio] wrong default output device");
               deleteOutParams();
             }
           }
@@ -277,7 +282,7 @@ void TrtAudio::updateAudioParams() {
           outDevId = 0;
       }
     } else {
-      qDebug() << "No audio devices!";
+      qDebug() << "[TrtAudio] No audio devices!";
       return;
     }
   // setting device parameters
@@ -310,13 +315,15 @@ void TrtAudio::updateAudioParams() {
       m_inSR = m_sampleRate;
       m_outSR = m_sampleRate;
     }
+#if !defined (Q_OS_MAC) // Mac has reaction for this flag - it open streams with 15 buffer frames
     streamOptions->flags |= RTAUDIO_MINIMIZE_LATENCY;
+#endif
     if (audioParams()->forwardInput && m_inParams && m_outParams)
         m_callBack = &passInputCallBack;
     else
         m_callBack = &duplexCallBack;
   }
-	ao()->emitParamsUpdated();
+  ao()->emitParamsUpdated();
 }
 
 
@@ -331,7 +338,7 @@ bool TrtAudio::listen() {
   }
   m_state = e_listening;
   rtDevice()->openStream(0, m_inParams, RTAUDIO_SINT16, m_inSR, &m_bufferFrames, &listenCallBack, 0, streamOptions);
-//   qDebug() << "stream is listening";
+//   qDebug() << "[TrtAudio] stream is listening";
   return true;
 }
 
@@ -346,14 +353,15 @@ bool TrtAudio::play() {
     }
   }
   m_state = e_playing;
+  m_sendPlayingFinished = true;
   rtDevice()->openStream(m_outParams, 0, RTAUDIO_SINT16, m_outSR, &m_bufferFrames, &playCallBack, 0, streamOptions);
-//   qDebug() << "stream is playing";
+//   qDebug() << "[TrtAudio] stream is playing";
   return true;
 }
 
 
 bool TrtAudio::openStream() {
-	try {
+  try {
     if (rtDevice()) {
       m_bufferFrames = m_preferredBF; // reset when it was overridden by another rt API
       if (m_areSplit) {
@@ -363,12 +371,12 @@ bool TrtAudio::openStream() {
         else
           splitTry = play();
         if (!splitTry) {
-          qDebug() << "Cannot open split stream";
+          qDebug() << "[TrtAudio] Cannot open split stream";
           return false;
         }
       } else if (!rtDevice()->isStreamOpen()) {
           rtDevice()->openStream(m_outParams, m_inParams, RTAUDIO_SINT16, sampleRate(), &m_bufferFrames, m_callBack, 0, streamOptions);
-          qDebug() << "audio opened in duplex mode";
+          qDebug() << "[TrtAudio] audio opened in duplex mode";
       }
 
       if (rtDevice()->isStreamOpen()) {
@@ -397,7 +405,7 @@ bool TrtAudio::openStream() {
         return false;
     }
   } catch (RtAudioError& e) {
-      qDebug() << "can't open stream" << m_inDevName << m_outDevName << "\n" << QString::fromStdString(e.getMessage());
+      qDebug() << "[TrtAudio] can't open stream" << m_inDevName << m_outDevName << "\n" << QString::fromStdString(e.getMessage());
       return false;
   }
   return true;
@@ -413,20 +421,20 @@ void TrtAudio::apiStopOrClose() {
 
 
 bool TrtAudio::startStream() {
-	if (!isOpened()) {
-		if (!openStream())
-			return false;
+  if (!isOpened()) {
+    if (!openStream())
+      return false;
   }
   try {
     if (rtDevice() && !rtDevice()->isStreamRunning()) {
       rtDevice()->startStream();
 //      qDebug("stream started");
-		}
+    }
   } catch (RtAudioError& e) {
-    qDebug() << "can't start stream";
-    return false;
+      qDebug() << "[TrtAudio] can't start stream";
+      return false;
   }
-//   qDebug("stream started");
+//   qDebug("[TrtAudio] stream started");
   return true;
 }
 
@@ -435,10 +443,10 @@ void TrtAudio::stopStream() {
   try {
     if (rtDevice() && rtDevice()->isStreamRunning()) {
       rtDevice()->stopStream();
-//       qDebug("stream stopped");
+//       qDebug("[TrtAudio] stream stopped");
     }
   } catch (RtAudioError& e) {
-    qDebug() << "can't stop stream";
+    qDebug() << "[TrtAudio] can't stop stream";
   }
 }
 
@@ -448,24 +456,24 @@ void TrtAudio::closeStream() {
     stopStream();
     if (rtDevice() && rtDevice()->isStreamOpen()) {
       rtDevice()->closeStream();
-      m_state = e_iddle;
-//       qDebug("stream closed");
+      m_state = e_idle;
+//       qDebug("[TrtAudio] stream closed");
     }
   } catch (RtAudioError& e) {
-    qDebug() << "can't close stream";
+      qDebug() << "[TrtAudio] can't close stream";
   }
 }
 
 
 void TrtAudio::abortStream() {
-	try {
+  try {
     if (rtDevice() && rtDevice()->isStreamRunning()) {
       rtDevice()->abortStream();
-//       qDebug("stream aborted");
+//       qDebug("[TrtAudio] stream aborted");
     }
   }
   catch (RtAudioError& e) {
-    qDebug() << "can't abort stream";
+    qDebug() << "[TrtAudio] can't abort stream";
   }
   if (areStreamsSplit())
     closeStream();
@@ -504,7 +512,7 @@ bool TrtAudio::getDeviceInfo(RtAudio::DeviceInfo& devInfo, int id) {
       devInfo = rtDevice()->getDeviceInfo(id);
   }
   catch (RtAudioError& e) {
-    qDebug() << "error when probing audio device" << id;
+    qDebug() << "[TrtAudio] error when probing audio device" << id;
     return false;
   }
   return true;
@@ -518,7 +526,7 @@ RtAudio::Api TrtAudio::getCurrentApi() {
         api = rtDevice()->getCurrentApi();
     }
     catch (RtAudioError& e) {
-      qDebug() << "Cannot determine current API";;
+      qDebug() << "[TrtAudio] Cannot determine current API";;
     }
   }
   return api;
@@ -532,7 +540,7 @@ unsigned int TrtAudio::getDeviceCount() {
         cnt = rtDevice()->getDeviceCount();
     }
     catch (RtAudioError& e) {
-      qDebug() << "Cannot obtain devices number";
+      qDebug() << "[TrtAudio] Cannot obtain devices number";
     }
   }
   return cnt;
@@ -546,7 +554,7 @@ int TrtAudio::getDefaultIn() {
       inNr = rtDevice()->getDefaultInputDevice();
     }
     catch (RtAudioError& e) {
-      qDebug() << "Cannot get default input device";
+      qDebug() << "[TrtAudio] Cannot get default input device";
     }
   }
   return inNr;
@@ -560,7 +568,7 @@ int TrtAudio::getDefaultOut() {
       inNr = rtDevice()->getDefaultOutputDevice();
     }
     catch (RtAudioError& e) {
-      qDebug() << "Cannot get default output device";
+      qDebug() << "[TrtAudio] Cannot get default output device";
     }
   }
   return inNr;
@@ -574,10 +582,10 @@ quint32 TrtAudio::determineSampleRate(RtAudio::DeviceInfo& devInfo) {
     if ( sr == 44100 || sr == 48000 || sr == 88200 || sr == 96000 || sr == 176400 || sr == 192000)
       return sr;
   }
-	if (devInfo.sampleRates.size() > 0)
-		return devInfo.sampleRates.at(devInfo.sampleRates.size() - 1);
-	else
-		return 44100;
+  if (devInfo.sampleRates.size() > 0)
+    return devInfo.sampleRates.at(devInfo.sampleRates.size() - 1);
+  else
+    return 44100;
 }
 
 #if defined(Q_OS_WIN)
@@ -587,9 +595,48 @@ void TrtAudio::restartASIO() {
 }
 #endif
 
+//#################################################################################################
+//###################              PRIVATE             ############################################
+//#################################################################################################
+int TrtAudio::duplexCallBack(void* outBuffer, void* inBuffer, unsigned int nBufferFrames, double, RtAudioStreamStatus status, void*) {
+  if (m_cbOut) {
+    if (m_cbOut(outBuffer, nBufferFrames, status))
+      if (m_cbIn)
+        m_cbIn(inBuffer, nBufferFrames, status);
+  } else
+      if (m_cbIn)
+        m_cbIn(inBuffer, nBufferFrames, status);
+  return 0;
+}
 
 
+int TrtAudio::passInputCallBack(void* outBuffer, void* inBuffer, unsigned int nBufferFrames, double, RtAudioStreamStatus status, void*) {
+  qint16 *in = (qint16*)inBuffer;
+  qint16 *out = (qint16*)outBuffer;
+  if (m_cbOut(outBuffer, nBufferFrames, status)) // none playing is performed
+      for (int i = 0; i < nBufferFrames; i++) { // then forward input
+          *out++ = *(in + i); // left channel
+          *out++ = *(in + i); // right channel
+      }
+  m_cbIn(inBuffer, nBufferFrames, status);
+  return 0;
+}
 
 
+int TrtAudio::playCallBack(void* outBuffer, void*, unsigned int nBufferFrames, double, RtAudioStreamStatus status, void*) {
+  if (m_cbOut(outBuffer, nBufferFrames, status)) {
+    if (m_sendPlayingFinished) {
+      m_sendPlayingFinished = false;
+      ao()->emitPlayingFinished();
+    }
+  }
+  return 0;
+}
+
+
+int TrtAudio::listenCallBack(void*, void* inBuffer, unsigned int nBufferFrames, double, RtAudioStreamStatus status, void*) {
+  m_cbIn(inBuffer, nBufferFrames, status);
+  return 0;
+}
 
 
