@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2011-2015 by Tomasz Bojczuk                             *
- *   tomaszbojczuk@gmail.com                                               *
+ *   Copyright (C) 2011-2016 by Tomasz Bojczuk                             *
+ *   seelook@gmail.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,15 +23,14 @@
 #include <exam/tlevel.h>
 #include <exam/textrans.h>
 #include <tnoofont.h>
+#include <tmtr.h>
 #include <widgets/tintonationview.h>
-#include <QtWidgets>
+#include <QtWidgets/QtWidgets>
 
 
-QString tableTip(const QString& tipText, TQAtype::Etype qType, TQAtype::Etype aType, int fSize) {
-	return "<table valign=\"middle\" align=\"center\"><tr><td>" + 
-	TnooFont::span(TquestionAsWdg::qaTypeSymbol(qType) + "?", fSize) + "</td>" + "<td align=\"center\">" + tipText + " </td> " +
-	"<td>" + TnooFont::span(TquestionAsWdg::qaTypeSymbol(aType) + "!", fSize) + "</td></tr></table>";
-}
+// TODO make order with layout code!!!
+// It works fine: more horizontal for desktop and more vertical for mobile
+// but it is hard to read and understand
 
 questionsSettings::questionsSettings(TlevelCreatorDlg* creator) :
     TabstractLevelPage(creator)
@@ -82,6 +81,9 @@ questionsSettings::questionsSettings(TlevelCreatorDlg* creator) :
 	QFont nf("nootka", fontMetrics().boundingRect("A").height());
 #if defined(Q_OS_MACX)
 	nf.setPointSize(fontMetrics().boundingRect("A").height() * 2);
+#elif defined (Q_OS_ANDROID)
+//   nf.setPointSize(qMax<int>(Tmtr::fingerPixels() / 3, fontMetrics().boundingRect("A").height()));
+  nf.setPointSize(Tmtr::fingerPixels() / 3);
 #endif
 	scoreNooLab->setFont(nf);
 	qaLay->addWidget(scoreNooLab, 2, 6, Qt::AlignCenter);
@@ -108,86 +110,86 @@ questionsSettings::questionsSettings(TlevelCreatorDlg* creator) :
 	m_qSoundNooLab->setFont(nf);
 	qaLay->addWidget(m_qSoundNooLab, 6, 5);
 	
-	QLabel *melodyLab = new QLabel(TnooFont::span("m", nootFontSize * 2, nooColor), this);
-	melodyLab->setAlignment(Qt::AlignCenter);
-	m_playMelodyChB = new QCheckBox(TexTrans::playMelodyTxt(), this);
-		m_playMelodyChB->setStatusTip(tableTip(TexTrans::playDescTxt(), TQAtype::e_asNote, TQAtype::e_asSound, nootFontSize));
-	m_writeMelodyChB = new QCheckBox(TexTrans::writeMelodyTxt(), this);
-		m_writeMelodyChB->setStatusTip(tableTip(TexTrans::writeDescTxt(), TQAtype::e_asSound, TQAtype::e_asNote, nootFontSize));
-// 		QCheckBox *m_repeatMelodyChB = new QCheckBox(tr("repeat melody"), this);
-	m_melodyLengthSpin = new QSpinBox(this);
-// 		m_melodyLengthSpin->setValue(5);
-		m_melodyLengthSpin->setMinimum(1);
-		m_melodyLengthSpin->setMaximum(50);
-		m_melodyLengthSpin->setStatusTip(tr("Maximum number of notes in a melody. Melody length is random value between 70% and 100% of that number."));
-	QLabel *lenghtLab = new QLabel(tr("Melody length"), this);
-		lenghtLab->setStatusTip(m_melodyLengthSpin->statusTip());
-		
-	m_finishOnTonicChB = new QCheckBox(tr("Melody ends on tonic note"), this);
-		m_finishOnTonicChB->setStatusTip(tr("Determines the last note of a melody.<br>When set, melody will be finished on tonic note of actual key signature."));
-	
 	m_tableWdg->setLayout(qaLay);
+  m_paintHandler = new TpaintHandler(widget());
+  m_paintHandler->setLayout(tabLay);
 	m_singleGr = new QGroupBox(tr("single note"), this);
 			m_singleGr->setCheckable(true);
-			m_singleGr->setLayout(tabLay);
-	
-	QVBoxLayout *melLay = new QVBoxLayout;
-		melLay->addWidget(melodyLab, 0, Qt::AlignCenter);
-		melLay->addStretch();
-		melLay->addWidget(m_playMelodyChB);
-		melLay->addWidget(m_writeMelodyChB);
-// 			melLay->addWidget(m_repeatMelodyChB);
-		melLay->addStretch();
-		QHBoxLayout *lenLay= new QHBoxLayout;
-			lenLay->addWidget(lenghtLab);
-			lenLay->addWidget(m_melodyLengthSpin);
-		melLay->addLayout(lenLay);
-		melLay->addWidget(m_finishOnTonicChB);
-		melLay->addStretch();
-	m_melodiesGr = new QGroupBox(tr("melodies"), this);
-			m_melodiesGr->setCheckable(true);
-			m_melodiesGr->setLayout(melLay);
-	QHBoxLayout *grBoxLay = new QHBoxLayout;
+      QVBoxLayout *outTabLay = new QVBoxLayout;
+      outTabLay->addWidget(m_paintHandler);
+      outTabLay->setContentsMargins(0, 0, 0, 0);
+			m_singleGr->setLayout(outTabLay);
+
+
+#if defined (Q_OS_ANDROID)
+    QVBoxLayout *grBoxLay = new QVBoxLayout;
+#else
+    auto grBoxLay = new QHBoxLayout;
+#endif
 		grBoxLay->addStretch();
 		grBoxLay->addWidget(m_singleGr);
 		grBoxLay->addStretch();
-		grBoxLay->addWidget(m_melodiesGr);
 		grBoxLay->addStretch();
-	
+
 	mainLay->addLayout(grBoxLay);
 	mainLay->addStretch();
-	
+
 // some checkBoxes
-	QGridLayout *chLay = new QGridLayout;
+#if defined (Q_OS_ANDROID)
+  auto chLay = new QVBoxLayout;
+#else
+	auto chLay = new QGridLayout;
+#endif
 	octaveRequiredChB = new QCheckBox(tr("require octave"),this);
 	octaveRequiredChB->setStatusTip(tr("if checked, selecting of valid octave is required"));
+#if defined (Q_OS_ANDROID)
+  chLay->addWidget(octaveRequiredChB, 0, Qt::AlignLeft);
+  chLay->addWidget(getLabelFromStatus(octaveRequiredChB, true, true));
+#else
 	chLay->addWidget(octaveRequiredChB, 0, 0, Qt::AlignLeft);
-	
+#endif
+
 	styleRequiredChB = new QCheckBox(tr("use different naming styles"),this);
 	styleRequiredChB->setStatusTip(tr("if checked, note names are switched between letters and solfege."));
+#if defined (Q_OS_ANDROID)
+  chLay->addWidget(styleRequiredChB, 0, Qt::AlignLeft);
+  chLay->addWidget(getLabelFromStatus(styleRequiredChB, true, true));
+#else
 	chLay->addWidget(styleRequiredChB, 1, 0, Qt::AlignLeft);
-	
+#endif
+
 	showStrNrChB = new QCheckBox(tr("show string number in questions"), this);
-	showStrNrChB->setStatusTip(tr("Shows on which string an answer has to be given.<br>Be careful, sometimes it is needed and sometimes it makes no sense."));
+	showStrNrChB->setStatusTip(
+      tr("Shows on which string an answer has to be given.<br>Be careful, sometimes it is needed and sometimes it makes no sense."));
+#if defined (Q_OS_ANDROID)
+  chLay->addWidget(showStrNrChB, 0, Qt::AlignLeft);
+  chLay->addWidget(getLabelFromStatus(showStrNrChB, true, true));
+#else
 	chLay->addWidget(showStrNrChB, 0, 1, Qt::AlignLeft);
+#endif
+
+  lowPosOnlyChBox = new QCheckBox(tr("notes in the lowest position only"),this);
+  lowPosOnlyChBox->setStatusTip(tr("if checked, the lowest position in selected fret range is required,<br>otherwise all possible positions of the note are acceptable.<br>To use this, all strings have to be available!"));
+#if defined (Q_OS_ANDROID)
+  chLay->addWidget(lowPosOnlyChBox, 0, Qt::AlignLeft);
+  chLay->addWidget(getLabelFromStatus(lowPosOnlyChBox, true, true));
+#else
+  chLay->addWidget(lowPosOnlyChBox, 1, 1, Qt::AlignLeft);
+#endif
+
 	mainLay->addLayout(chLay);
-	
-	lowPosOnlyChBox = new QCheckBox(tr("notes in the lowest position only"),this);
-	lowPosOnlyChBox->setStatusTip(tr("if checked, the lowest position in selected fret range is required,<br>otherwise all possible positions of the note are acceptable.<br>To use this, all strings have to be available!"));
-	chLay->addWidget(lowPosOnlyChBox, 1, 1, Qt::AlignLeft);
-	
+
 	TintonationCombo *intoCombo = new TintonationCombo(this);
 	m_intonationCombo = intoCombo->accuracyCombo; // we need only combo box (label is not necessary)
 	mainLay->addWidget(intoCombo, 0, Qt::AlignCenter);
-	
-		
+
+#if defined (Q_OS_ANDROID) // add space at the bottom to allow touching lower boxes and combo
+  mainLay->addSpacing(Tmtr::fingerPixels() / 2);
+  mainLay->setContentsMargins(0, 10, 0, 10);
+#endif
 	setLayout(mainLay);
 
-  connect(m_singleGr, SIGNAL(clicked()), this, SLOT(singleMultiSlot()));
-  connect(m_melodiesGr, SIGNAL(clicked()), this, SLOT(singleMultiSlot()));
-  connect(m_playMelodyChB, SIGNAL(clicked()), this, SLOT(melodyQuestionSlot()));
-  connect(m_writeMelodyChB, SIGNAL(clicked()), this, SLOT(melodyQuestionSlot()));
-  
+  connect(m_singleGr, SIGNAL(clicked()), this, SLOT(whenParamsChanged()));
 	connect(asNoteWdg, SIGNAL(answerStateChanged()), this, SLOT(whenParamsChanged()));
 	connect(asNameWdg, SIGNAL(answerStateChanged()), this, SLOT(whenParamsChanged()));
 	connect(asFretPosWdg, SIGNAL(answerStateChanged()), this, SLOT(whenParamsChanged()));
@@ -198,10 +200,7 @@ questionsSettings::questionsSettings(TlevelCreatorDlg* creator) :
 	connect(showStrNrChB, SIGNAL(clicked()), this, SLOT(whenParamsChanged()));
 	connect(lowPosOnlyChBox, SIGNAL(clicked()), this, SLOT(whenParamsChanged()));
 	connect(m_intonationCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(whenParamsChanged()));
-	connect(m_playMelodyChB, SIGNAL(clicked()), this, SLOT(whenParamsChanged()));
-	connect(m_writeMelodyChB, SIGNAL(clicked()), this, SLOT(whenParamsChanged()));
-	connect(m_melodyLengthSpin, SIGNAL(valueChanged(int)), this, SLOT(whenParamsChanged()));
-	
+	connect(m_paintHandler, &TpaintHandler::paintMe, this, &questionsSettings::paintSlot);
 }
 
 //#################################################################################################
@@ -209,7 +208,7 @@ questionsSettings::questionsSettings(TlevelCreatorDlg* creator) :
 //#################################################################################################
 
 void questionsSettings::loadLevel(Tlevel* level) {
-	blockSignals(true);
+  blockSignals(true);
     asNoteWdg->setAnswers(level->answersAs[TQAtype::e_asNote]);
     asNoteWdg->setChecked(level->questionAs.isNote()); // when it is false it cleans all checkBoxes to false
     asNameWdg->setAnswers(level->answersAs[TQAtype::e_asName]);
@@ -218,42 +217,22 @@ void questionsSettings::loadLevel(Tlevel* level) {
     asFretPosWdg->setChecked(level->questionAs.isFret());
     asSoundWdg->setAnswers(level->answersAs[TQAtype::e_asSound]);
     asSoundWdg->setChecked(level->questionAs.isSound());
-    
+
     octaveRequiredChB->setChecked(level->requireOctave);
     styleRequiredChB->setChecked(level->requireStyle);
     showStrNrChB->setChecked(level->showStrNr);
     lowPosOnlyChBox->setChecked(level->onlyLowPos);
-		m_intonationCombo->setCurrentIndex(level->intonation);
-		if (level->melodyLen == 1)
-			m_melodyLengthSpin->setRange(1, 1);
-		else
-			m_melodyLengthSpin->setRange(2, 50);
-		m_melodyLengthSpin->setValue(level->melodyLen);
-		m_finishOnTonicChB->setChecked(level->endsOnTonic);
-		saveLevel(wLevel());
-		if (wLevel()->canBeMelody()) {
-			m_melodiesGr->setChecked(true);
-			m_singleGr->setChecked(false);
-			if (level->questionAs.isNote() && level->answersAs[TQAtype::e_asNote].isSound())
-				m_playMelodyChB->setChecked(true);
-			else
-				m_playMelodyChB->setChecked(false);
-			if (level->questionAs.isSound() && level->answersAs[TQAtype::e_asSound].isNote())
-				m_writeMelodyChB->setChecked(true);
-			else
-				m_writeMelodyChB->setChecked(false);
-		} else {
-				m_singleGr->setChecked(true);
-				m_melodiesGr->setChecked(false);
-				m_playMelodyChB->setChecked(false);
-				m_writeMelodyChB->setChecked(false);
-		}
-	blockSignals(false);
+    m_intonationCombo->setCurrentIndex(level->intonation);
+    m_singleGr->setChecked(level->melodyLen == 1);
+    saveLevel(workLevel());
+//     adjustToLevel();
+  blockSignals(false);
   adjustToLevel();
 }
 
 
 void questionsSettings::saveLevel(Tlevel* level) {
+//   qDebug() << "[questionsSettings] save level" << (level == workLevel() ? QStringLiteral("work") : level->name);
   level->questionAs.setAsNote(asNoteWdg->isChecked());
   level->answersAs[TQAtype::e_asNote] = asNoteWdg->getAnswers();
   level->questionAs.setAsName(asNameWdg->isChecked());
@@ -262,41 +241,29 @@ void questionsSettings::saveLevel(Tlevel* level) {
   level->answersAs[TQAtype::e_asFretPos] = asFretPosWdg->getAnswers();
   level->questionAs.setAsSound(asSoundWdg->isChecked());
   level->answersAs[TQAtype::e_asSound] = asSoundWdg->getAnswers();
-  
+
   level->requireOctave = octaveRequiredChB->isChecked();
   level->requireStyle = styleRequiredChB->isChecked();
   level->showStrNr = showStrNrChB->isChecked();
   level->onlyLowPos = lowPosOnlyChBox->isChecked();
   level->intonation = m_intonationCombo->currentIndex();
-  
-  level->melodyLen = m_melodyLengthSpin->value();
-  level->endsOnTonic = m_finishOnTonicChB->isChecked();
 }
 
 
 void questionsSettings::setMelodiesEnabled(bool enableMelodies) {
-  m_singleGr->blockSignals(true);
-  m_melodiesGr->blockSignals(true);
+  const QSignalBlocker blocker(m_singleGr);
   m_singleGr->setChecked(!enableMelodies);
-  m_melodiesGr->setChecked(enableMelodies);
   if (enableMelodies) {
     asNameWdg->setChecked(false);
     asFretPosWdg->setChecked(false);
-    m_melodyLengthSpin->setRange(2, 50);
-  } else {
-    m_playMelodyChB->setChecked(false);
-    m_writeMelodyChB->setChecked(false);
-    m_melodyLengthSpin->setRange(1, 1);
   }
   asSoundWdg->setChecked(false); // reset it either for melodies or for single note
   asNoteWdg->setChecked(false); // reset it either for melodies or for single note
-  m_singleGr->blockSignals(false);
-  m_melodiesGr->blockSignals(false);
 }
 
 
 void questionsSettings::changed() {
-
+  loadLevel(workLevel());
 }
 
 
@@ -323,15 +290,12 @@ void questionsSettings::adjustToLevel() {
     showStrNrChB->setDisabled(lowDisabled);
   }
 // Is sound input enabled to allow intonation check
-  if ((m_singleGr->isChecked() &&
-    (asNoteWdg->answerAsSound() || asNameWdg->answerAsSound() || asFretPosWdg->answerAsSound() || asSoundWdg->answerAsSound())) ||
-    (m_melodiesGr->isChecked() && m_playMelodyChB->isChecked()) )
+  if (asNoteWdg->answerAsSound() || asNameWdg->answerAsSound() || asFretPosWdg->answerAsSound() || asSoundWdg->answerAsSound())
       m_intonationCombo->setDisabled(false);
   else
       m_intonationCombo->setDisabled(true);
 // Disable name styles when no name as question or answer
-  if (m_melodiesGr->isChecked() ||
-      !(asNameWdg->isChecked() || asNoteWdg->answerAsName() || asFretPosWdg->answerAsName() || asSoundWdg->answerAsName())) {
+  if (!(asNameWdg->isChecked() || asNoteWdg->answerAsName() || asFretPosWdg->answerAsName() || asSoundWdg->answerAsName())) {
       styleRequiredChB->setChecked(false);
       styleRequiredChB->setDisabled(true);
   } else {
@@ -343,6 +307,12 @@ void questionsSettings::adjustToLevel() {
 
 void questionsSettings::whenParamsChanged() {
   if (!signalsBlocked()) {
+    if (sender() == m_singleGr) {
+      if (m_singleGr->isChecked())
+        workLevel()->melodyLen = 1;
+      else
+        workLevel()->melodyLen = 2;
+    }
     adjustToLevel();
     changedLocal();
   }
@@ -367,8 +337,8 @@ void questionsSettings::hideGuitarRelated() {
 }
 
 
-void questionsSettings::paintEvent(QPaintEvent* ) {
-  QPainter painter(this);
+void questionsSettings::paintSlot( ) {
+  QPainter painter(m_paintHandler);
   QPen pen = painter.pen();
 	if (m_singleGr->isChecked())
 		pen.setColor(palette().color(QPalette::Active, QPalette::Text));
@@ -377,15 +347,14 @@ void questionsSettings::paintEvent(QPaintEvent* ) {
   pen.setWidth(1);
 	pen.setStyle(Qt::DashLine);
   painter.setPen(pen);
-	QPoint tl = m_singleGr->mapToParent(m_singleGr->contentsRect().topLeft());
-  int vertLineUpY = tl.y() + m_questLab->geometry().y() + m_questLab->geometry().height() + 14;
-  painter.drawLine(tl.x() + 10, vertLineUpY, tl.x() + m_singleGr->contentsRect().width() - 20, vertLineUpY);
-  int vertLineDownY = tl.y() + asSoundWdg->enableChBox->geometry().y() + asSoundWdg->enableChBox->geometry().height() + 14;
-  painter.drawLine(tl.x() + 10, vertLineDownY, tl.x() + m_singleGr->contentsRect().width() - 20, vertLineDownY);
-  int horLineLeftX = tl.x() + asNoteWdg->enableChBox->x()	+ asNoteWdg->enableChBox->width() + 14;
-  painter.drawLine(horLineLeftX, tl.y() + 10, horLineLeftX, tl.y() + m_singleGr->contentsRect().height() - 20);
-  int horLineRightX = tl.x() + m_asSoundLab->geometry().x() + m_asSoundLab->geometry().width() + 14;
-  painter.drawLine(horLineRightX, tl.y() + 10, horLineRightX, tl.y() + m_singleGr->contentsRect().height() - 20);
+  int vertLineUpY = m_questLab->geometry().y() + m_questLab->geometry().height() + 14;
+  painter.drawLine(10, vertLineUpY, m_singleGr->contentsRect().width() - 20, vertLineUpY);
+  int vertLineDownY =asSoundWdg->enableChBox->geometry().y() + asSoundWdg->enableChBox->geometry().height() + 14;
+  painter.drawLine(10, vertLineDownY, m_singleGr->contentsRect().width() - 20, vertLineDownY);
+  int horLineLeftX = asNoteWdg->enableChBox->x()	+ asNoteWdg->enableChBox->width() + 14;
+  painter.drawLine(horLineLeftX,10, horLineLeftX,m_singleGr->contentsRect().height() - 20);
+  int horLineRightX = m_asSoundLab->geometry().x() + m_asSoundLab->geometry().width() + 14;
+  painter.drawLine(horLineRightX,10, horLineRightX,m_singleGr->contentsRect().height() - 20);
 	
 }
 
@@ -400,25 +369,6 @@ void questionsSettings::stringsCheckedSlot(bool checked) {
 }
 
 
-void questionsSettings::singleMultiSlot() {
-	setMelodiesEnabled((sender() == m_singleGr && !m_singleGr->isChecked()) || (sender() == m_melodiesGr && m_melodiesGr->isChecked()));
-}
-
-
-void questionsSettings::melodyQuestionSlot() {
-	if (m_playMelodyChB->isChecked()) {
-			asNoteWdg->setAnswers(TQAtype(false, false, false, true));
-			asNoteWdg->setChecked(true);
-	} else {
-			asNoteWdg->setChecked(false);
-	}
-	if (m_writeMelodyChB->isChecked()) {
-			asSoundWdg->setAnswers(TQAtype(true, false, false, false));
-			asSoundWdg->setChecked(true);
-	} else {
-			asSoundWdg->setChecked(false);
-	}
-}
 
 
 
